@@ -1,7 +1,7 @@
 # Copyright Notice:
 # Copyright 2016-2021 DMTF. All rights reserved.
 # License: BSD 3-Clause License. For full text see link:
-# https://github.com/DMTF/python-redfish-library/blob/master/LICENSE.md
+# https://github.com/chhuang789
 
 # -*- coding: utf-8 -*-
 """Helper module for working with REST technology."""
@@ -78,10 +78,11 @@ class redfish_advantech:
         self.port = port
         self.username = username
         self.password = password
+        self.payload = None
         self.theTimeout = 10
-        self.connection = any
-        self.authToken = ''
-        self.location = ''
+        self.connection = None
+        self.authToken = None
+        self.location = None
         self.url = ''
         self.method = ''
         self.urlThermal = ''
@@ -97,11 +98,25 @@ class redfish_advantech:
         self.nCount = 0
         self.nIndex = 0
         self.urlLogEntries = ''
+        self.__logVerbose = 0  # 1111
 
     def log(self, msg):
         self.logger.info("%s [hostname=%s port%d]",
                          msg, self.hostname, self.port)
 
+    def get_logVerbose(self):  # 1111
+        """Return the level of log verbose"""
+        return self.__logVerbose
+
+    def set_logVerbose(self, logVerbose=0):  # 1111
+        """Set log Verbose level
+
+        :param logVerbose: The level of log verbose to be set.
+        :type logVerbose: int
+
+        """
+        self.__logVerbose = logVerbose
+
     def __del__(self):
         self.logger.info('=== Destroy of redfish_advantech.__del__ ===')
 
@@ -115,7 +130,7 @@ class redfish_advantech:
 
     # Redfish http request
     def rfRequest(self, log=True):
-        if (self.authToken == ''):  # for login only
+        if (self.authToken == None):  # for login only
             headers = {'Accept': '*/*',
                        'self.connection': 'Keep-Alive', 'OData-Version': '4.0'}
         else:  # for other requests
@@ -125,25 +140,22 @@ class redfish_advantech:
             if (log):
                 self.logger.info(
                     "--> rfRequest [%s %s]", self.method, self.url)
-                if (self.payload == ''):
-                    self.logger.debug("headers=%s", headers)
+                if (self.payload == None):
+                    if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 0):
+                        self.logger.debug("headers=%s", headers)
                 else:
-                    self.logger.debug("headers=%s", headers)
-                    self.logger.debug("self.payload=%s", self.payload)
-            if (self.payload == ''):
+                    if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 0):
+                        self.logger.debug("headers=%s", headers)
+                        self.logger.debug("self.payload=%s", self.payload)
+            if (self.payload == None):
                 self.connection.request(
                     self.method, self.url, self.payload, headers)
             else:
                 self.connection.request(
                     self.method, self.url, json.dumps(self.payload), headers)
             response = self.connection.getresponse()
-        # except:
-        #    self.logger.error(
-        #        'Unknown exception when create the http self.connection and exit()')
-        #    exit()
         except Exception as e:
             self.logger.error(e)
-            exit()
 
         if (log):
             self.logger.info("response.status(reason)=%d(%s)",
@@ -179,9 +191,15 @@ class redfish_advantech:
         self.logger.info("--> Login [%s %s]", self.method, self.url)
         self.connection = http.client.HTTPSConnection(
             self.hostname, self.port, timeout=self.theTimeout)
-        self.payload = {'UserName': self.username, 'Password': self.password}
+        data = dict()
+        data['UserName'] = self.username
+        data['Password'] = self.password
+        self.payload = data
+
         response = self.rfRequest(self)
         result = response.read().decode(errors='replace')
+        if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+            self.logger.debug("result=%s", result)
         # Get Token and Location of session after login
         self.authToken = response.headers['X-Auth-Token']
         self.logger.info("--> X-Auth-Token=%s]", self.authToken)
@@ -192,43 +210,58 @@ class redfish_advantech:
                 if i[0] == '@odata.id':
                     self.location = i[1]
                     self.logger.info("location=%s", self.location)
+        self.payload = None
 
     # Logout
     def logout(self):
         """ Logout of session. YOU MUST CALL THIS WHEN YOU ARE DONE TO FREE UP SESSIONS"""
-        if (self.authToken != ''):
+        if (self.authToken):
             self.url = self.location
             self.method = "DELETE"
             self.logger.info("--> Logout [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
-            self.logger.info("response.status(reason)=%d(%s)",
+            if response.status not in [200, 202, 204]:
+                self.logger.info("Invalid session resource: %s, return code: %d" % (
+                    self.url, response.status))
+            self.logger.info("User logout response.status(reason)=%d(%s)",
                              response.status, response.reason)
+            self.authToken = None
+            self.location = None
 
     # Disconnect
     def disconnect(self):
         self.logger.debug("---> Disconnect http self.connection")
-        try:
-            ret = self.connection.close()
-            if (ret == None):
-                self.logger.info('http self.connection closed successfully')
-            else:
-                logging.error('http self.connection closed failed with ', ret)
-        except:
-            logging.error(
-                'Unknown exception when close the http self.connection')
-        self.logger.info('=== End to of redfish_advantech.disconnect ===')
+        if (self.connection):
+            try:
+                ret = self.connection.close()
+                if (ret == None):
+                    self.logger.info(
+                        'http self.connection closed successfully')
+                else:
+                    logging.error(
+                        'http self.connection closed failed with ', ret)
+            except:
+                logging.error(
+                    'Unknown exception when close the http self.connection')
+        else:
+            self.logger.info(
+                'http self.connection is not connected. No need to close it.')
+        self.connection = None
+        self.logger.debug('=== End to of redfish_advantech.disconnect ===')
 
     # Get Chassis
     def getChassis(self):
         self.method = "GET"
         self.url = "/redfish/v1/Chassis"
         self.logger.info("--> getChassis [%s %s]", self.method, self.url)
-        self.payload = ''
+        self.payload = None
         response = self.rfRequest()
         # Get the next link of Chassis
         self.url = ''
         result = response.read().decode(errors='replace')
+        if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+            self.logger.debug("result=%s", result)
         if (response.getcode() == 200):
             json_data = json.loads(result)
             for i in json_data.items():
@@ -244,12 +277,14 @@ class redfish_advantech:
 
     # Get Chassis/1u
     def getChassis1u(self):
-        if not (self.url == ''):
+        if (self.url != ''):
             self.method = "GET"
-            self.payload = ''
+            self.payload = None
             self.logger.info("--> getChassis1u [%s %s]", self.method, self.url)
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug("result=%s", result)
             if (response.getcode() == 200):
                 json_data = json.loads(result)
                 for i in json_data.items():
@@ -273,14 +308,16 @@ class redfish_advantech:
 
     # Get Chassis/1u/Thermal
     def getChassis1uThermal(self):
-        if not (self.urlThermal == ''):
+        if (self.urlThermal != ''):
             self.method = "GET"
             self.url = self.urlThermal
             self.logger.info(
                 "--> getChassis1uThermal [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
             if (response.getcode() == 200):
                 json_data = json.loads(result)
                 for i in json_data.items():
@@ -313,14 +350,16 @@ class redfish_advantech:
 
     # Get Chassis/1u/Power
     def getChassis1uPower(self):
-        if not (self.urlPower == ''):
+        if (self.urlPower != ''):
             self.method = "GET"
             self.url = self.urlPower
             self.logger.info(
                 "--> getChassis1uPower [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
             if (response.getcode() == 200):
                 json_data = json.loads(result)
                 for i in json_data.items():
@@ -356,9 +395,11 @@ class redfish_advantech:
         self.url = "/redfish/v1/Systems"
         self.method = "GET"
         self.logger.info("--> getSystems [%s %s]", self.method, self.url)
-        self.payload = ''
+        self.payload = None
         response = self.rfRequest()
         result = response.read().decode(errors='replace')
+        if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+            self.logger.debug(result)
         # Get the next link of Systems
         self.url = ''
         if (response.getcode() == 200):
@@ -376,14 +417,15 @@ class redfish_advantech:
 
     # Get Systems/0
     def getSystems0(self):
-        if not (self.url == ''):
+        if (self.url != ''):
             self.url = "/redfish/v1/Systems/0"
             self.method = "GET"
             self.logger.debug("--> getSystems0 [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
-
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
             if (response.getcode() == 200):
                 json_data = json.loads(result)
                 for i in json_data.items():
@@ -441,14 +483,16 @@ class redfish_advantech:
 
     # Get Systems/0/Bios
     def getSystems0Bios(self):
-        if not (self.urlBios == ''):
+        if (self.urlBios != ''):
             self.url = self.urlBios
             self.method = "GET"
             self.logger.info(
                 "--> getSystems0Bios [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
             # Get contents of Systems/Bios
             if (response.getcode() == 200):
                 json_data = json.loads(result)
@@ -457,14 +501,16 @@ class redfish_advantech:
 
     # Get Systems/0/Processors
     def getSystems0Processors(self):
-        if not (self.urlProcessors == ''):
+        if (self.urlProcessors != ''):
             self.url = self.urlProcessors
             self.method = "GET"
             self.logger.info(
                 "--> getSystems0Processors [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
             # Get contents of Systems/Bios
             if (response.getcode() == 200):
                 json_data = json.loads(result)
@@ -487,13 +533,15 @@ class redfish_advantech:
 
     # Get Systems/0/Processors/CPU0
     def getSystems0ProcessorsCPU0(self):
-        if not (self.url == ''):
+        if (self.url != ''):
             self.method = "GET"
             self.logger.info(
                 "--> getSystems0ProcessorsCPU0 [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
             # Get contents of Systems/Bios
             if (response.getcode() == 200):
                 json_data = json.loads(result)
@@ -502,14 +550,16 @@ class redfish_advantech:
 
     # Get Systems/0/SimpleStorage
     def getSystems0SimpleStorage(self):
-        if not (self.urlSimpleStorage == ''):
+        if (self.urlSimpleStorage != ''):
             self.url = self.urlSimpleStorage
             self.method = "GET"
             self.logger.debug(
                 "--> getSystems0SimpleStorage [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
             # Get contents of Systems/Bios
             if (response.getcode() == 200):
                 json_data = json.loads(result)
@@ -536,14 +586,16 @@ class redfish_advantech:
     # Get Systems/0/SimpleStorage/*
     def getSystems0SimpleStorageAll(self):
         for i in range(self.nCount):
-            if not (self.lstURL[i] == ''):
+            if (self.lstURL[i] != ''):
                 self.url = self.lstURL[i]
                 self.method = "GET"
                 self.logger.debug(
                     "--> getSystems0SimpleStorageAll [%s %s]", self.method, self.url)
-                self.payload = ''
+                self.payload = None
                 response = self.rfRequest()
                 result = response.read().decode(errors='replace')
+                if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                    self.logger.debug(result)
                 if (response.getcode() == 200):
                     json_data = json.loads(result)
                     for i in json_data.items():
@@ -551,14 +603,16 @@ class redfish_advantech:
 
     # Get Systems/0/Memory
     def getSystems0Memory(self):
-        if not (self.urlMemory == ''):
+        if (self.urlMemory != ''):
             self.url = self.urlMemory
             self.method = "GET"
             self.logger.info(
                 "--> getSystems0Memory [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
             if (response.getcode() == 200):
                 json_data = json.loads(result)
                 for i in json_data.items():
@@ -587,14 +641,16 @@ class redfish_advantech:
     # Get Systems/0/Memory/*
     def getSystems0MemoryAll(self):
         for i in range(self.nCount):
-            if not (self.lstURL[i] == ''):
+            if (self.lstURL[i] != ''):
                 self.url = self.lstURL[i]
                 self.method = "GET"
                 self.logger.info(
                     "--> getSystems0MemoryAll [%s %s]", self.method, self.url)
-                self.payload = ''
+                self.payload = None
                 response = self.rfRequest()
                 result = response.read().decode(errors='replace')
+                if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                    self.logger.debug(result)
                 if (response.getcode() == 200):
                     json_data = json.loads(result)
                     for i in json_data.items():
@@ -602,12 +658,12 @@ class redfish_advantech:
 
     # Get Systems/0/EthernetInterfaces
     def getSystems0EthernetInterfaces(self):
-        if not (self.urlEthernetInterfaces == ''):
+        if (self.urlEthernetInterfaces != ''):
             self.url = self.urlEthernetInterfaces
             self.method = "GET"
             self.logger.info(
                 "--> getSystems0EthernetInterfaces [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
             if (response.getcode() == 200):
@@ -638,14 +694,16 @@ class redfish_advantech:
     # Get Systems/0/EthernetInterfaces/*
     def getSystems0EthernetInterfacesAll(self):
         for i in range(self.nCount):
-            if not (self.lstURL[i] == ''):
+            if (self.lstURL[i] != ''):
                 self.url = self.lstURL[i]
                 self.method = "GET"
                 self.logger.info(
                     "--> getSystems0EthernetInterfacesAll [%s %s]", self.method, self.url)
-                self.payload = ''
+                self.payload = None
                 response = self.rfRequest()
                 result = response.read().decode(errors='replace')
+                if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                    self.logger.debug(result)
                 if (response.getcode() == 200):
                     json_data = json.loads(result)
                     for i in json_data.items():
@@ -653,14 +711,16 @@ class redfish_advantech:
 
     # Get Systems/0/LogServices
     def getSystems0LogServices(self):
-        if not (self.urlLogServices == ''):
+        if (self.urlLogServices != ''):
             self.url = self.urlLogServices
             self.method = "GET"
             self.logger.info(
                 "--> getSystems0LogServices [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
             if (response.getcode() == 200):
                 json_data = json.loads(result)
                 for i in json_data.items():
@@ -690,14 +750,16 @@ class redfish_advantech:
     def getSystems0LogServicesLog(self):
         self.urlLogEntries = ''
         for i in range(self.nCount):
-            if not (self.lstURL[i] == ''):
+            if (self.lstURL[i] != ''):
                 self.url = self.lstURL[i]
                 self.method = "GET"
                 self.logger.info(
                     "--> getSystems0LogServicesLog [%s %s]", self.method, self.url)
-                self.payload = ''
+                self.payload = None
                 response = self.rfRequest()
                 result = response.read().decode(errors='replace')
+                if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                    self.logger.debug(result)
                 if (response.getcode() == 200):
                     json_data = json.loads(result)
                     for i in json_data.items():
@@ -713,18 +775,20 @@ class redfish_advantech:
 
     # Get Systems/0/LogServices/Log/Entries
     def getSystems0LogServicesLogEntries(self):
-        if not (self.urlLogEntries == ''):
+        if (self.urlLogEntries != ''):
             self.url = self.urlLogEntries
             self.method = "GET"
             self.logger.info(
                 "--> getSystems0LogServicesLogEntries [%s %s]", self.method, self.url)
-            self.payload = ''
+            self.payload = None
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 2):
+                self.logger.debug(result)
             if (response.getcode() == 200):
                 json_data = json.loads(result)
                 for i in json_data.items():
-                    if not (i[0] == "Members"):
+                    if (i[0] != "Members"):
                         self.logger.debug("%s: %s", i[0], i[1])
         # Get the next link(s) of Entries
         self.lstURL = []
@@ -744,35 +808,49 @@ class redfish_advantech:
                             if iii[0] == '@odata.id':
                                 self.lstURL.append(iii[1])
                                 self.nIndex = self.nIndex + 1
-                                self.logger.debug(
-                                    "Next link=%s", self.lstURL[self.nIndex-1])
+                                if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 2):
+                                    self.logger.debug(
+                                        "Next link=%s", self.lstURL[self.nIndex-1])
                 else:
                     self.logger.debug("%s: %s", i[0], i[1])
         # Get Systems/0/LogServices/Log/Entries/*
         for i in range(self.nCount):
-            if not (self.lstURL[i] == ''):
+            if (self.lstURL[i] != ''):
                 self.url = self.lstURL[i]
                 self.method = "GET"
-                self.payload = ''
+                self.payload = None
                 response = self.rfRequest(False)
                 result = response.read().decode(errors='replace')
+                if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 2):
+                    self.logger.debug(result)
                 if (response.getcode() == 200):
                     json_data = json.loads(result)
                     for i in json_data.items():
-                        self.logger.debug("%s: %s", i[0], i[1])
+                        if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 2):
+                            self.logger.debug("%s: %s", i[0], i[1])
 
     # GracefulShutdown or Power on
     def actionGracefulShutdownOrPowerOn(self):
-        if not (self.strPowerState == ''):
+        if (self.strPowerState != ''):
             self.url = "/redfish/v1/Systems/0/Actions/ComputerSystem.Reset"
             self.method = "POST"
             self.logger.info(
                 "--> actionGracefulShutdownOrPowerOn [%s %s]", self.method, self.url)
             if (self.strPowerState == 'On'):
-                self.payload = {'ResetType': 'GracefulShutdown'}
+                #self.payload = {'ResetType': 'GracefulShutdown'}
+                data = dict()
+                data['ResetType'] = 'GracefulShutdown'
+                self.payload = data
                 self.logger.info('self.payload GracefulShutdown')
             else:
-                self.payload = {'ResetType': 'On'}
+                #self.payload = {'ResetType': 'On'}
+                data = dict()
+                data['ResetType'] = 'On'
+                self.payload = data
                 self.logger.info('self.payload Power On')
             response = self.rfRequest()
             result = response.read().decode(errors='replace')
+            if (self.logger.isEnabledFor(logging.DEBUG) and self.get_logVerbose() >= 1):
+                self.logger.debug(result)
+        self.payload = None
+
